# -*- coding: UTF-8 -*-

#   icd_make_plan.py    202011.21   kita

#!! 軸と任意点（もしくは方向）を指示して組立平面を作成し断面を表示する

# 2021.4.11     円弧を指示して断面軸を指示できるように改良
# 2023.11.23    「Top」「Front」「Rside」作成ボタン追加


import clr                  # .NETをimport
clr.AddReference('sxnet')   # 
from sxnet import *     # 全てのクラスをimport
SxSys.init(3999)

import tkinter as tk
import tkinter.ttk as ttk       # ComboBoxを使うのに必要
import numpy as np
import math
import os

my_fn= os.path.basename(__file__)       # icd_make_plan.py
save_fn= my_fn.replace('.py','.dat')    # icd_make_plan.dat

plan = []
idx = 0


def make_pln_3p(v_0, v_x, v_y):  # 組立平面作成 p[原点, x方向点,y方向点] 3点指示
    global plan, idx
    #print(idx)
    pos = SxPos()   # 原点
    pos.x = v_0[0]; pos.y = v_0[1] ; pos.z = v_0[2]
    
    zvec = SxVec()  # 組立平面法線方向ベクトル
    zvec0 = np.cross(v_x,v_y)     # 法線ベクトル 外積
    zvec.x = zvec0[0];  zvec.y = zvec0[1];  zvec.z = zvec0[2]

    xvec = SxVec()  # 組立平面X方向ベクトル
    xvec.x = v_x[0]; xvec.y = v_x[1]; xvec.z = v_x[2]
    
    #print(pos.x, pos.y, pos.z)
    #print(zvec.x, zvec.y, zvec.z)
    #print(xvec.x, xvec.y, xvec.z)

    #plan = SxAsmPlane.create(pos, zvec , xvec)
    tmp = SxAsmPlane.create(pos, zvec, xvec)
        # 組立平面原点座標, 組立平面法線方向ベクトル, 組立平面X軸方向ベクトル
    plan.append(tmp)
    idx = idx + 1
    #ss = ';@CRSDSP 1'   # 断面表示
    ss = ';@CRSON'   # 断面表示
    SxSys.command(ss,False)
    #return plan    


def push_exe():    # 任意軸による作成
    axis = v1.get()     # 回転軸
    drct = v2.get()     # 方向　direction
    ang_rad = float(e_ang.get()) * np.pi/180   # 角度[rad]
    
    v00 = np.array([0.0, 0.0, 0.0])
    if axis == 0: # X軸
        v02 = np.array([1.0, 0.0, 0.0])
    elif axis == 1: # Y軸
        v02 = np.array([0.0, 1.0, 0.0])
    elif axis == 2: # Z軸
        v02 = np.array([0.0, 0.0, 1.0])
    elif axis == 3: # 任意軸
        print('\n  ワイヤー線か円弧を選択してください')
        opt = SxOptSelect()
        opt.select_ent = True
        opt.select_pos = False 
        opt.select_go = True
        opt.select_edge = True      # エッジの選択を可能にする

        inf = SxSys.select(opt)     # マウスピック
        #print(inf.status)
        if inf.status == SxInfSelect.STATUS_ENT:       # ワイヤsegment 選択
            geom = inf.ent.getGeom()   # 選択された要素(Segment)を返す
            #leng = geom.leng
            sp = geom.sp         # SxPos
            vec = geom.vec       # SxVec (単位ベクトル）
            v00 = np.array([sp.x, sp.y, sp.z])
            v02 = np.array([vec.x, vec.y, vec.z])
        elif inf.status == SxInfSelect.STATUS_EDGE: #エッジ選択
            geom = inf.edge.getGeom()   # 選択された要素(Segment)を返す
            if geom.type == SxGeom.TYPE_LINE3D or geom.type == SxGeom.TYPE_OTHER_CURVE:
                print('\n  対象外の要素です\007\n')
                return
            #leng = geom.leng
            else:  #円弧　or 円
                cp = geom.cp         # SxPos
                vec = geom.axis      # SxVec (法線単位ベクトル）
                v00 = np.array([cp.x, cp.y, cp.z])
                v02 = np.array([vec.x, vec.y, vec.z])

        else:

            print('\n  ワイヤー線が選択されませんでした \n\007')
            return

    if drct == 0: #'X方向':
        v01 = np.array([1.0, 0.0, 0.0])
    elif drct == 1: #'Y方向':
        v01 = np.array([0.0, 1.0, 0.0])
    elif drct == 2: #'Z方向':
        v01 = np.array([0.0, 0.0, 1.0])
    elif drct == 3: #'任意方向':
        print('\n  任意の点を選択してください \007\n')
        opt = SxOptSelect()
        opt.select_ent = False
        opt.select_pos = True 
        opt.select_go = True
        inf = SxSys.select(opt)     # マウスピック
        p1_ = inf.pos
        v01 = np.array([p1_.x, p1_.y, p1_.z]) - v00 

    elif drct == 4: # 数値
        if axis == 0: # X軸
            v01 = np.array([0.0, np.sin(ang_rad), np.cos(ang_rad)])
        elif axis == 1: # Y軸
            v01 = np.array([np.cos(ang_rad), 0.0, -np.sin(ang_rad)])
        elif axis == 2: # Z軸
            v01 = np.array([np.cos(ang_rad), np.sin(ang_rad), 0.0])
        elif axis == 3: # 任意軸
            v01 = np.array([np.cos(ang_rad), 0.0, -np.sin(ang_rad)])

    
    th =  math.acos(np.dot(v01,v02) / np.linalg.norm(v01) / np.linalg.norm(v02))
    v_x = v01 - v02/np.linalg.norm(v02) * np.linalg.norm(v01) * np.cos(th)
    v0 = v00; v_y = v02
    
    #print(v0, v_x, v_y)
    try:
        make_pln_3p(v0, v_x, v_y)     
    except:
        print('\n  軸と方向が同じです \n\007')
    #make_pln_3p(v0, v_x, v_y)     

        
def push_del():     # 組立平面削除
    global plan, idx
    #print(idx)
    try:
        #plan.delete()  #組立平面削除
        idx -= 1
        plan[idx].delete()  #組立平面削除
    except:
        print('\n  削除できる平面がありません \n\007')

# 組立平面作成
def push_make_TFR(): # Top, Front, Rside作成
    view_list = SxModel.getActive().getVSList()
    name_list= []
    for view in view_list:
      inf = view.getInf()  
      print(inf.name)
      name_list.append(inf.name)
    
    # 組立平面開設
    # Top
    if not ('Top' in name_list or 'TOP' in name_list):
        #input('Plan Top がないので作成します')
        #quit()
        pos= SxPos(0.0, 0.0, 0.0)
        zvec= SxVec(0.0, 100.0, 0.0)    # 法線ベクトル y
        xvec= SxVec(100.0, 0.0, 0.0)    # x方向ベクトル
        plan_top= SxAsmPlane.create(pos, zvec, xvec)
        SxVS.getActive().setName("Top")

    # Front
    if not ('Front' in name_list or 'FRONT' in name_list): 
        zvec= SxVec(0.0, 0.0, 100.0)    # 法線ベクトル z
        xvec= SxVec(100.0, 0.0, 0.0)    # x方向ベクトル
        plan_front= SxAsmPlane.create(pos, zvec, xvec)
        SxVS.getActive().setName("Front")
    
    # Rside
    if not ('Rside' in name_list or 'RSIDE' in name_list): 
        zvec= SxVec(100.0, 0.0, 0.0)    # 法線ベクトル z
        xvec= SxVec(0.0, 0.0, -100.0)    # -z方向ベクトル
        plan_rside= SxAsmPlane.create(pos, zvec, xvec)
        SxVS.getActive().setName("Rside")



#************************************
# ボタン等の部品配置
#************************************
nl = 8
x00 = 5; y00 = 5; dy = 26

root = tk.Tk()
root.attributes("-topmost", True)    # 常に前面に表示
root.configure(bg='RoyalBlue4')
root.title(my_fn)   #タイトル
#root.title('組立平面作成 Ver.0.8')   #タイトル
root.geometry('160x%d+1000+0' % (nl*dy-10))
#root.attributes("-toolwindow",1)    # 最小、最大化ボタン非表示

btn_bcol = 'grey21'     #ボタン背景色
btn_tcol = 'white'      #ボタン文字色
label_bcol = 'khaki3'   #ラベル背景色
label_tcol = 'black'   #ラベル文字色
e_bcol = 'khaki4'




#-------------------------------------------
ss = '実行'; x0= 5; y0= dy*5+y00
#------------------------------------------
btn_exe = tk.Button(root,text=ss, command = push_exe, bg = btn_bcol, fg = btn_tcol)  # Buttonの作成
btn_exe.place(x=x0,y=y0)

#-------------------------------------------
# 数値入力エントリー部品
#-------------------------------------------
x0 = 80
e_ang = tk.Entry(root, width=5,justify="right", bg=e_bcol)  # エントリー部品の生成
e_ang.place(x=x0,y=y0)
e_ang.insert(tk.END,'30')
ss = '[deg]'
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x0+40,y=y0)


#-------------------------------------------
ss = 'T,F,Rs作成'; x0 = 5; y0= dy*6+y00
#------------------------------------------
btn_TFR = tk.Button(root,text=ss, command = push_make_TFR, bg = 'orange', fg = 'black')
btn_TFR.place(x=x0,y=y0)


#-------------------------------------------
ss = '組平削除'; x0 = 5; y0 = dy*4+y00
#-------------------------------------------
btn_exe = tk.Button(root,text=ss, command = push_del, bg = btn_bcol, fg = btn_tcol)  # Buttonの作成
btn_exe.place(x=x0,y=y0)

#-------------------------------------------
ss = '回転軸'; x0 = 5; y0 = 5   #ラジオボタン
#-------------------------------------------
v1 = tk.IntVar(root, value = 3)    #初期値設定
f1 = ttk.LabelFrame(root, text = ss)        #frame作成
ss1 = ['X軸','Y軸','Z軸','任意軸']
for i in range(len(ss1)):      # iには 0,1,2,3が入る
    r = ttk.Radiobutton(f1, text = ss1[i], value = i, variable = v1)
    r.pack() 
f1.place(x=x0,y=y0)

#-------------------------------------------
ss = 'X方向'; x0 = 80; y0 = 5   #ラジオボタン
#-------------------------------------------
v2 = tk.IntVar(root, value = 3)    #初期値設定
f2 = ttk.LabelFrame(root, text = ss)        #frame作成
ss2 = ['X方向','Y方向','Z方向','任意方向','数値']
for i in range(len(ss2)):      # iには 0,1,2,3,4が入る
    r = ttk.Radiobutton(f2, text = ss2[i], value = i, variable = v2)
    r.pack() 
f2.place(x=x0,y=y0)



root.mainloop()
